#ifndef ANNEAL_HH
#define ANNEAL_HH

#include "Vector.hh"

typedef enum {GEOMETRIC, FAST} ReductionSchedule;
typedef enum {METROPOLIS, SZU} AcceptanceCriteria;

void perturb(const Vector& point, Vector& newpoint, double temp);

class Anneal {
  typedef double (*funcPtr)(const Vector&);
  typedef void (*perturbPtr)(const Vector&, Vector&, double);

public:
  Anneal(funcPtr            objFunc,
	 perturbPtr         pertFunc = perturb,
	 ReductionSchedule  reduce   = GEOMETRIC,
	 AcceptanceCriteria accept   = METROPOLIS,
	 int                numTemps = 200,
	 int                numIters = 5);
  double anneal(Vector& point);
  ~Anneal();

private:
  funcPtr            objFunc;
  perturbPtr         pertFunc;
  ReductionSchedule  reduce;
  AcceptanceCriteria accept;
  int                numTemps;
  int                numIters;
};

#endif
