#ifndef COVARIANCE_HH
#define COVARIANCE_HH

#include "Vector.hh"
#include "Matrix.hh"
#include "SymmetricMatrix.hh"

class Covariance : public SymmetricMatrix {
public:
  Covariance(int dimensions);
  virtual int getNumParams() const = 0;
  virtual void eval(const Vector& param) = 0;
  virtual double pred(const Vector& x, const Vector& param, Vector& k) const = 0;
  virtual ~Covariance();
}; 

class ARD {
public:
  ARD(const Matrix& data);
  virtual ~ARD();

protected:
  double ard(const Vector& p, const Vector& q, const Vector& param) const;
  const Matrix& train;
  int v;
};

class ARDLinear : public ARD {
public:
  ARDLinear(const Matrix& data);
  virtual ~ARDLinear();

protected:
  double linear(const Vector& p, const Vector& q, const Vector& param) const;
  int a0, a1;
};

class ARDLinearNoise : public Covariance, public ARDLinear {
public:
  ARDLinearNoise(const Matrix& data);
  int getNumParams() const;
  void eval(const Vector& param);
  double pred(const Vector& x, const Vector& param, Vector& k) const;
  virtual ~ARDLinearNoise();

private:
  int sigma;
};

class ARDLinearARCH : public Covariance, public ARDLinear {
public:
  ARDLinearARCH(const Matrix& data, double s0);
  int getNumParams() const;
  void eval(const Vector& param);
  double pred(const Vector& x, const Vector& param, Vector& k) const;
  virtual ~ARDLinearARCH();

private:
  double sigma0;
  int omega, beta;
};

class ARDLinearQuadGARCH : public Covariance, public ARDLinear {
public:
  ARDLinearQuadGARCH(const Matrix& data, const Vector& targets, double s0);
  int getNumParams() const;
  void eval(const Vector& param);
  double pred(const Vector& x, const Vector& param, Vector& k) const;
  virtual ~ARDLinearQuadGARCH();

private:
  const Vector& targ;
  double sigma0;
  int omega, alpha, beta;
};

class ARDLinearQuadAGARCH : public Covariance, public ARDLinear {
public:
  ARDLinearQuadAGARCH(const Matrix& data, const Vector& targets, double s0);
  int getNumParams() const;
  void eval(const Vector& param);
  double pred(const Vector& x, const Vector& param, Vector& k) const;
  virtual ~ARDLinearQuadAGARCH();

private:
  const Vector& targ;
  double sigma0;
  int omega, alpha, rho, beta;
};

#endif
