#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include "Gnuplot2d.hh"

Gnuplot2d::Gnuplot2d() {
  fp = popen("gnuplot", "w");
  if(fp == 0) {
    cerr << "error: failed to open gnuplot" << endl;
    exit(1);
  }
}

void
Gnuplot2d::setTerminal(Terminal term, const char* fileName = 0) {
  if(fileName != 0)
    fprintf(fp, "set output \"%s\"\n", fileName);
  switch(term) {
  case X11:
    fprintf(fp, "set terminal x11\n");
    break;
  case LATEX:
    fprintf(fp, "set terminal latex\n");
    break;
  case POSTSCRIPT:
    fprintf(fp, "set terminal postscript eps\n");
    break;
  }
  fflush(fp);
}    

void 
Gnuplot2d::setStyle(PlotStyle sty) {
  switch(sty) {
  case LINES:
    fprintf(fp, "set data style lines\n");
    break;
  case POINTS:
    fprintf(fp, "set data style points\n");
    break;
  case LINESPOINTS:
    fprintf(fp, "set data style linespoints\n");
    break;
  case DOTS:
    fprintf(fp, "set data style dots\n");
    break;
  }
  fflush(fp);
}

void
Gnuplot2d::setTitle(const char* str) {
  fprintf(fp, "set title \"%s\"\n", str);
  fflush(fp);
}

void
Gnuplot2d::setXLabel(const char* str) {
  fprintf(fp, "set xlabel \"%s\"\n", str);
  fflush(fp);
}

void
Gnuplot2d::setYLabel(const char* str) {
  fprintf(fp, "set ylabel \"%s\"\n", str);
  fflush(fp);
}

void 
Gnuplot2d::setXRange(double min, double max) {
  fprintf(fp, "set xrange [%g:%g]\n", min, max);
  fflush(fp);
}

void 
Gnuplot2d::setYRange(double min, double max) {
  fprintf(fp, "set yrange [%g:%g]\n", min, max);
  fflush(fp);
}

void 
Gnuplot2d::setGrid(Boolean b) {
  if(b == TRUE)
    fprintf(fp, "set grid\n");
  else
    fprintf(fp, "set nogrid\n");
  fflush(fp);
}

void 
Gnuplot2d::plot(const Vector& x, const Vector& y) {
  char fn[64];
  if(mkfifo(tmpnam(fn), 777) != 0) {
    cerr << "error: failed to make FIFO named pipe" << endl;
    exit(1);
  }
  fprintf(fp, "set nokey\n");
  fprintf(fp, "plot \"%s\"\n", fn);
  fflush(fp);
  FILE* fifo = fopen(fn, "w");
  for(int i = 0; i < x.getSize(); i++)
    fprintf(fifo, "%g\t%g\n", x[i], y[i]);
  fclose(fifo);
  unlink(fn);
}

void 
Gnuplot2d::plot(const Vector& x, const Vector& y, const Vector& error) {
  char fn[64];
  if(mkfifo(tmpnam(fn), 777) != 0) {
    cerr << "error: failed to make FIFO named pipe" << endl;
    exit(1);
  }
  fprintf(fp, "set nokey\n");
  fprintf(fp, "set data style errorbars\n");
  fprintf(fp, "plot \"%s\"\n", fn);
  fflush(fp);
  FILE* fifo = fopen(fn, "w");
  for(int i = 0; i < x.getSize(); i++)
    fprintf(fifo, "%g\t%g\t%g\n", x[i], y[i], error[i]);
  fclose(fifo);
  unlink(fn);
}

void 
Gnuplot2d::plot(const Vector& x0, const Vector& y0, const Vector& x1, const Vector& y1) {
  char fn0[64], fn1[64];
  if(mkfifo(tmpnam(fn0), 777) != 0 || mkfifo(tmpnam(fn1), 777) != 0) {
    cerr << "error: failed to make FIFO named pipe" << endl;
    exit(1);
  }
  fprintf(fp, "set nokey\n");
  fprintf(fp, "plot \"%s\", \"%s\"\n", fn0, fn1);
  fflush(fp);
  FILE* fifo = fopen(fn0, "w");
  for(int i = 0; i < x0.getSize(); i++)
    fprintf(fifo, "%g\t%g\n", x0[i], y0[i]);
  fclose(fifo);
  unlink(fn0);
  fifo = fopen(fn1, "w");
  for(int i = 0; i < x1.getSize(); i++)
    fprintf(fifo, "%g\t%g\n", x1[i], y1[i]);
  fclose(fifo);
  unlink(fn1);
}

void
Gnuplot2d::message(const char* msg) {
  char c;
  cout << msg << endl;
  cout << "Press <ENTER> to kill plot" << endl;
  cin.get(c);
}

Gnuplot2d::~Gnuplot2d() {
  pclose(fp);
}
