#include <cassert>
#include "Matrix.hh"

inline
int
Matrix::getRows() const {
  return rows;
}

inline
int
Matrix::getCols() const {
  return cols;
}

inline
const
Vector&
Matrix::operator[](int i) const {
#ifdef DEBUG
  assert(i >= 0 && i < rows);
#endif
  return vecPtr[i];
}

inline
Vector&
Matrix::operator[](int i) {
#ifdef DEBUG
  assert(i >= 0 && i < rows);
#endif
  return vecPtr[i];
}

Matrix::Matrix(int row, int col) :
  rows(row), cols(col), vecPtr(new Vector[rows]) {
#ifdef DEBUG
  assert(rows >= 0 && cols >= 0);
#endif
  for(int i = 0; i < rows; i++)
    (*this)[i].resize(cols);
}

Matrix::Matrix(const Matrix& matrix) :
  rows(matrix.rows), cols(matrix.cols), vecPtr(new Vector[rows]) {
  for(int i = 0; i < rows; i++) {
    (*this)[i].resize(cols);
    (*this)[i] = matrix[i];
  }
}

Matrix::Matrix(const SymmetricMatrix& matrix) :
  rows(matrix.getDime()), cols(matrix.getDime()), vecPtr(new Vector[rows]) {
  for(int i = 0; i < rows; i++) {
    (*this)[i].resize(cols);
    for(int j = 0; j < cols; j++)
      (*this)[i][j] = matrix(i, j);
  }
}

Matrix&
Matrix::operator=(const Matrix& matrix) {
#ifdef DEBUG
  assert(rows == matrix.rows);
  assert(cols == matrix.cols);
#endif
  if(this != &matrix)
    for(int i = 0; i < rows; i++)
      (*this)[i] = matrix[i];
  return *this;
}

Matrix&
Matrix::operator=(double value) {
  for(int i = 0; i < rows; i++)
    (*this)[i] = value;
  return *this;
}

Vector
Matrix::operator*(const Vector& vector) const {
#ifdef DEBUG
  assert(cols == vector.getSize());
#endif
  Vector vec(rows);
  for(int i = 0; i < rows; i++)
    vec[i] = (*this)[i].dotProduct(vector);
  return vec;
}

Vector 
Matrix::row(int i) const {
#ifdef DEBUG
  assert(i >= 0 && i < rows);
#endif
  return (*this)[i];
}

Vector
Matrix::col(int j) const {
#ifdef DEBUG
  assert(j >= 0 && j < cols);
#endif
  Vector vec(rows);
  for(int i = 0; i < rows; i++)
    vec[i] = (*this)[i][j];
  return vec;
}

void 
Matrix::row(int i, const Vector& vector) {
#ifdef DEBUG
  assert(i >= 0 && i < rows);
  assert(cols == vector.getSize());
#endif
  (*this)[i] = vector;
}

void 
Matrix::col(int j, const Vector& vector) {
#ifdef DEBUG
  assert(j >= 0 && j < cols);
  assert(rows == vector.getSize());
#endif
  for(int i = 0; i < rows; i++)
    (*this)[i][j] = vector[i];
}

Matrix::~Matrix() {
  delete [] vecPtr;
}

ostream&
operator<<(ostream& oStream, const Matrix& matrix) {
  for(int i = 0; i < matrix.rows; i++)
    oStream << matrix[i] << endl;
  return oStream;
}

istream&
operator>>(istream& iStream, Matrix& matrix) {
  for(int i = 0; i < matrix.rows; i++)
    iStream >> matrix[i];
  return iStream;
}
