#include "Posterior.hh"

MinusLogLike::MinusLogLike(Covariance& c, const Vector& t) :
  cov(c), targets(t) {
}

inline
double
MinusLogLike::minusLogLike(const Vector& theta) {
  cov.eval(theta);
  double logDet = cov.choleskyDecomposeInvert();
  return 0.5 * (logDet + cov.vecXMatXvec(targets));
}

MinusLogLike::~MinusLogLike() {
}

IAMinusLogLike::IAMinusLogLike(Covariance& c, const Vector& t) :
  cov(c), targets(t) {
}

inline
double
IAMinusLogLike::minusLogLike(const Vector& theta) {
  cov.eval(theta);
  Vector verr(cov.getDime());
  Vector xhat(cov.getDime());
  double logDet = cov.innovations(targets, xhat, verr);
  return 0.5 * (logDet + targets.squaredNorm(xhat, verr));
}

IAMinusLogLike::~IAMinusLogLike() {
}

MinusLogPrior::MinusLogPrior() {
}

inline 
double
MinusLogPrior::minusLogPrior(const Vector& theta) {
  return 0.0;
}

MinusLogPrior::~MinusLogPrior() {
}
