#ifndef SYMMETRIC_MATRIX_HH
#define SYMMETRIC_MATRIX_HH

#include <iostream.h>
#include "Vector.hh"

class NotPosDefException {
public:
  NotPosDefException() {}
  ~NotPosDefException() {}
};

class SymmetricMatrix {
public:
  SymmetricMatrix(int dimensions);
  SymmetricMatrix(const SymmetricMatrix& matrix);

  SymmetricMatrix& operator=(const SymmetricMatrix& matrix);
  SymmetricMatrix& operator=(double value);

  double choleskyDecompose();
  double choleskyDecomposeInvert();
  double innovations(const Vector& x, Vector& p, Vector& v);
  double traceProduct(const SymmetricMatrix& matrix) const;
  double vecXMatXvec(const Vector& vector) const;

  double operator()(int i, int j) const;
  double& operator()(int i, int j);

  void vecXMat(const Vector& vector, Vector& result) const;

  int getDime() const;

  virtual ~SymmetricMatrix();

  friend ostream& operator<<(ostream& oStream, const SymmetricMatrix& matrix);
  friend istream& operator>>(istream& iStream, SymmetricMatrix& matrix);

protected:
  const Vector& operator[](int i) const;
  Vector& operator[](int i);

  int dime;       // dimensions of symmetric matrix
  Vector* vecPtr; // symmetric matrix is an array of vectors
};

#endif
