#include <iostream.h>
#include "Math.hh"
#include "Anneal.hh"
#include "Gnuplot2d.hh"

double func1(const Vector& v);
double func2(const Vector& v);
double f_sphere(const Vector& x);
double f_Griew(const Vector& x);
double shekel(const Vector& x);
double Micha(const Vector& x);
double langerman(const Vector& x);
double SqrDst(const Vector& x1, double x2[]);

void main(void) {
  Vector v(5, 0.0);
  {
    Anneal an(func1);
    cout << "func1 Off by: "    << Math::abs(an.anneal(v) - 1.0E+3) << endl;
  }
  v = 0.0;
  {
    Anneal an(func2);
    cout << "func2 Off by: "    << Math::abs(an.anneal(v) - 1.0E+3) << endl;
  }
  v = 0.0;
  {
    Anneal an(f_sphere);
    cout << "f_sphere Off by: " << Math::abs(an.anneal(v) - 1.0E-6) << endl;
  }
  v = 0.0;
  {
    Anneal an(f_Griew);
    cout << "f_Griew Off by: "  << Math::abs(an.anneal(v) - 1.0E-4) << endl;
  }
  v = 0.0;
  {
    Anneal an(shekel);
    cout << "shekel Off by: "   << Math::abs(an.anneal(v) + 9)      << endl;
  }
  v = 0.0;
  {
    Anneal an(Micha);
    cout << "Micha Off by: "    << Math::abs(an.anneal(v) + 4.687)  << endl;
  }
  v = 0.0;
  {
    Anneal an(langerman);
    cout << "langerman Off by: "<< Math::abs(an.anneal(v) + 1.4)    << endl;
  }
}

//
// Value to reach: 1.0E+3
//
double
func1(const Vector& v) {
  static double wid[100] = {1.0, 3.0, 10.0, 30.0};
  static double off[100] = {111.0, -222.0, 333.0, -444.0, 999.0};
  double q, r, sumd = 0.0, sumr = 0.0;
  for(int i = 0; i < v.getSize(); i++) {
    q = (v[i] - off[i]) * wid[i];
    r = (double)(q >= 0.0 ? (int)(q + 0.5) : -(int)(0.5 - q));
    sumr += q * q;
    sumd += (q - r) * (q - r);
  }
  return (off[4] + sumr * (1.0 + (sumd > 0.09 ? 2.0 : 2.0 * sumd / 0.09)));
}

//
// Value to reach: 1.0E+3
//
double
func2(const Vector& v) {
  static double off[100] = {111.0, -222.0, 333.0, -444.0, 999.0};
  double sum1 = 0.0, sum2 = 0.0;
  for(int i = 0; i < v.getSize(); i++) {
    sum1 += (v[i] - off[i]);
    sum2 += Math::sqr(v[i] - off[i]);
  }
  return off[4] + (Math::sqr(20.0 * sin(sum1)) + sum2);
}

//
// Value to reach: 1.0E-6
//
double
f_sphere(const Vector& x) {
  int i;
  double Sum;

  for (Sum = 0.0, i = 0; i < x.getSize(); i++) {
    Sum += (x[i]-1.0)*(x[i]-1.0);	
  }

  return (Sum);
}

//
// Value to reach: 1.0E-4
//
double
f_Griew(const Vector& x) {
  static double D = 4000.0;
  int i;
  double Val1, Val2, Sum;

  for (Val1 = 0.0, Val2 = 1.0, i = 0; i < x.getSize(); i++) {
    Val1 += (x[i]-100.0) * (x[i]-100.0);
    Val2 *= cos((x[i]-100.0) / sqrt((double) (i + 1)));
  }

  Sum = Val1 / D - Val2 + 1.0;
  return (Sum);
}

//
// Value to reach: -9
//
double
shekel(const Vector& x) {

  static double a[30][10] = {
    {9.681, 0.667, 4.783, 9.095, 3.517, 9.325, 6.544, 0.211, 5.122, 2.020},
    {9.400, 2.041, 3.788, 7.931, 2.882, 2.672, 3.568, 1.284, 7.033, 7.374},
    {8.025, 9.152, 5.114, 7.621, 4.564, 4.711, 2.996, 6.126, 0.734, 4.982},
    {2.196, 0.415, 5.649, 6.979, 9.510, 9.166, 6.304, 6.054, 9.377, 1.426},
    {8.074, 8.777, 3.467, 1.863, 6.708, 6.349, 4.534, 0.276, 7.633, 1.567},
    {7.650, 5.658, 0.720, 2.764, 3.278, 5.283, 7.474, 6.274, 1.409, 8.208},
    {1.256, 3.605, 8.623, 6.905, 0.584, 8.133, 6.071, 6.888, 4.187, 5.448},
    {8.314, 2.261, 4.224, 1.781, 4.124, 0.932, 8.129, 8.658, 1.208, 5.762},
    {0.226, 8.858, 1.420, 0.945, 1.622, 4.698, 6.228, 9.096, 0.972, 7.637},
    {7.305, 2.228, 1.242, 5.928, 9.133, 1.826, 4.060, 5.204, 8.713, 8.247},
    {0.652, 7.027, 0.508, 4.876, 8.807, 4.632, 5.808, 6.937, 3.291, 7.016},
    {2.699, 3.516, 5.874, 4.119, 4.461, 7.496, 8.817, 0.690, 6.593, 9.789},
    {8.327, 3.897, 2.017, 9.570, 9.825, 1.150, 1.395, 3.885, 6.354, 0.109},
    {2.132, 7.006, 7.136, 2.641, 1.882, 5.943, 7.273, 7.691, 2.880, 0.564},
    {4.707, 5.579, 4.080, 0.581, 9.698, 8.542, 8.077, 8.515, 9.231, 4.670},
    {8.304, 7.559, 8.567, 0.322, 7.128, 8.392, 1.472, 8.524, 2.277, 7.826},
    {8.632, 4.409, 4.832, 5.768, 7.050, 6.715, 1.711, 4.323, 4.405, 4.591},
    {4.887, 9.112, 0.170, 8.967, 9.693, 9.867, 7.508, 7.770, 8.382, 6.740},
    {2.440, 6.686, 4.299, 1.007, 7.008, 1.427, 9.398, 8.480, 9.950, 1.675},
    {6.306, 8.583, 6.084, 1.138, 4.350, 3.134, 7.853, 6.061, 7.457, 2.258},
    {0.652, 2.343, 1.370, 0.821, 1.310, 1.063, 0.689, 8.819, 8.833, 9.070},
    {5.558, 1.272, 5.756, 9.857, 2.279, 2.764, 1.284, 1.677, 1.244, 1.234},
    {3.352, 7.549, 9.817, 9.437, 8.687, 4.167, 2.570, 6.540, 0.228, 0.027},
    {8.798, 0.880, 2.370, 0.168, 1.701, 3.680, 1.231, 2.390, 2.499, 0.064},
    {1.460, 8.057, 1.336, 7.217, 7.914, 3.615, 9.981, 9.198, 5.292, 1.224},
    {0.432, 8.645, 8.774, 0.249, 8.081, 7.461, 4.416, 0.652, 4.002, 4.644},
    {0.679, 2.800, 5.523, 3.049, 2.968, 7.225, 6.730, 4.199, 9.614, 9.229},
    {4.263, 1.074, 7.286, 5.599, 8.291, 5.200, 9.214, 8.272, 4.398, 4.506},
    {9.496, 4.830, 3.150, 8.270, 5.079, 1.231, 5.731, 9.494, 1.883, 9.732},
    {4.138, 2.562, 2.532, 9.661, 5.611, 5.500, 6.886, 2.341, 9.699, 6.500}};

  static double c[] = {
    0.806,
    0.517,
    0.1,
    0.908,
    0.965,
    0.669,
    0.524,
    0.902,
    0.531,
    0.876,
    0.462,
    0.491,
    0.463,
    0.714,
    0.352,
    0.869,
    0.813,
    0.811,
    0.828,
    0.964,
    0.789,
    0.360,
    0.369,
    0.992,
    0.332,
    0.817,
    0.632,
    0.883,
    0.608,
    0.326};

  int i, j;
  double sp, h, result = 0.0;

  for (i = 0; i < 30; i++) {
    sp = 0.0;
    for (j = 0; j < x.getSize(); j++) {
      h   = x[j] - a[i][j];
      sp += h * h;
    }
    result += 1.0 / (sp + c[i]);
  }
  return( - result);
}

//
// Values to reach: -4.687 for the 5 dim, -9.66 for the 10 dim
//
double Micha(const Vector& x) {
  static double m = 10.0;
  double u;
  int i;

  u = 0.0; 
  for (i = 0; i < x.getSize(); i++) 
    u = u + sin(x[i]) * pow(sin((i+1) * x[i] * x[i] / Math::pi()), 2.0 * m); 

  return(-u); 
}

//
// Value to reach: -1.4
//
double
langerman(const Vector& x) {

  static double a[30][10] = {
    {9.681, 0.667, 4.783, 9.095, 3.517, 9.325, 6.544, 0.211, 5.122, 2.020},
    {9.400, 2.041, 3.788, 7.931, 2.882, 2.672, 3.568, 1.284, 7.033, 7.374},
    {8.025, 9.152, 5.114, 7.621, 4.564, 4.711, 2.996, 6.126, 0.734, 4.982},
    {2.196, 0.415, 5.649, 6.979, 9.510, 9.166, 6.304, 6.054, 9.377, 1.426},
    {8.074, 8.777, 3.467, 1.863, 6.708, 6.349, 4.534, 0.276, 7.633, 1.567},
    {7.650, 5.658, 0.720, 2.764, 3.278, 5.283, 7.474, 6.274, 1.409, 8.208},
    {1.256, 3.605, 8.623, 6.905, 0.584, 8.133, 6.071, 6.888, 4.187, 5.448},
    {8.314, 2.261, 4.224, 1.781, 4.124, 0.932, 8.129, 8.658, 1.208, 5.762},
    {0.226, 8.858, 1.420, 0.945, 1.622, 4.698, 6.228, 9.096, 0.972, 7.637},
    {7.305, 2.228, 1.242, 5.928, 9.133, 1.826, 4.060, 5.204, 8.713, 8.247},
    {0.652, 7.027, 0.508, 4.876, 8.807, 4.632, 5.808, 6.937, 3.291, 7.016},
    {2.699, 3.516, 5.874, 4.119, 4.461, 7.496, 8.817, 0.690, 6.593, 9.789},
    {8.327, 3.897, 2.017, 9.570, 9.825, 1.150, 1.395, 3.885, 6.354, 0.109},
    {2.132, 7.006, 7.136, 2.641, 1.882, 5.943, 7.273, 7.691, 2.880, 0.564},
    {4.707, 5.579, 4.080, 0.581, 9.698, 8.542, 8.077, 8.515, 9.231, 4.670},
    {8.304, 7.559, 8.567, 0.322, 7.128, 8.392, 1.472, 8.524, 2.277, 7.826},
    {8.632, 4.409, 4.832, 5.768, 7.050, 6.715, 1.711, 4.323, 4.405, 4.591},
    {4.887, 9.112, 0.170, 8.967, 9.693, 9.867, 7.508, 7.770, 8.382, 6.740},
    {2.440, 6.686, 4.299, 1.007, 7.008, 1.427, 9.398, 8.480, 9.950, 1.675},
    {6.306, 8.583, 6.084, 1.138, 4.350, 3.134, 7.853, 6.061, 7.457, 2.258},
    {0.652, 2.343, 1.370, 0.821, 1.310, 1.063, 0.689, 8.819, 8.833, 9.070},
    {5.558, 1.272, 5.756, 9.857, 2.279, 2.764, 1.284, 1.677, 1.244, 1.234},
    {3.352, 7.549, 9.817, 9.437, 8.687, 4.167, 2.570, 6.540, 0.228, 0.027},
    {8.798, 0.880, 2.370, 0.168, 1.701, 3.680, 1.231, 2.390, 2.499, 0.064},
    {1.460, 8.057, 1.336, 7.217, 7.914, 3.615, 9.981, 9.198, 5.292, 1.224},
    {0.432, 8.645, 8.774, 0.249, 8.081, 7.461, 4.416, 0.652, 4.002, 4.644},
    {0.679, 2.800, 5.523, 3.049, 2.968, 7.225, 6.730, 4.199, 9.614, 9.229},
    {4.263, 1.074, 7.286, 5.599, 8.291, 5.200, 9.214, 8.272, 4.398, 4.506},
    {9.496, 4.830, 3.150, 8.270, 5.079, 1.231, 5.731, 9.494, 1.883, 9.732},
    {4.138, 2.562, 2.532, 9.661, 5.611, 5.500, 6.886, 2.341, 9.699, 6.500}};

  static double c[] = {
    0.806,
    0.517,
    1.5,
    0.908,
    0.965,
    0.669,
    0.524,
    0.902,
    0.531,
    0.876,
    0.462,
    0.491,
    0.463,
    0.714,
    0.352,
    0.869,
    0.813,
    0.811,
    0.828,
    0.964,
    0.789,
    0.360,
    0.369,
    0.992,
    0.332,
    0.817,
    0.632,
    0.883,
    0.608,
    0.326};

  int i;
  double Sum, dist;
	
  Sum = 0.0;
  for(i = 0; i < 5; i++) {
    dist = SqrDst(x, a[i]);
    Sum -= c[i] * (exp(-dist / Math::pi()) * cos(Math::pi() * dist));
  }

  return (Sum);
}

double
SqrDst(const Vector& x1, double x2[]) {
  double dist = 0.0, d;
  int i;

  dist = 0.0;
  for (i = 0; i< x1.getSize(); i++) {
    d = x1[i] - x2[i];
    dist += d * d;
  }

  return (dist);
}
